<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Context;

use Dedi\SyliusQuotePlugin\Exception\QuoteNotFoundException;
use Dedi\SyliusQuotePlugin\Specification\Quote\IsCartSpecification;
use Dedi\SyliusQuotePlugin\Storage\QuoteCartStorageInterface;
use Sylius\Component\Channel\Context\ChannelContextInterface;
use Sylius\Component\Channel\Context\ChannelNotFoundException;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Order\Model\OrderInterface;

class SessionAndChannelBasedQuoteCartContext implements QuoteCartContextInterface
{
    private readonly ChannelContextInterface $channelContext;

    private IsCartSpecification $isCartSpecification;

    private readonly QuoteCartStorageInterface $quoteStorage;

    public function __construct(
        ChannelContextInterface $channelContext,
        QuoteCartStorageInterface $quoteStorage,
        IsCartSpecification $isCartSpecification,
    ) {
        $this->channelContext = $channelContext;
        $this->quoteStorage = $quoteStorage;
        $this->isCartSpecification = $isCartSpecification;
    }

    public function getCart(): OrderInterface
    {
        try {
            /** @var ChannelInterface $channel */
            $channel = $this->channelContext->getChannel();
        } catch (ChannelNotFoundException $exception) {
            throw new QuoteNotFoundException(null, $exception);
        }

        if (!$this->quoteStorage->hasForChannel($channel)) {
            throw new QuoteNotFoundException('Sylius was not able to find the quote in session');
        }

        $quote = $this->quoteStorage->getForChannel($channel);
        if (null === $quote || !$this->isCartSpecification->isSatisfiedBy($quote)) {
            $this->quoteStorage->removeForChannel($channel);

            throw new QuoteNotFoundException('Sylius was not able to find the quote in session');
        }

        return $quote;
    }
}
